<?php
/*
Plugin Name: ryoumou-trade-agent-Access-Log
Description: 訪問者のアクセス元情報を解析し、リファラや記事ごとのアクセス数を表示します。
Version: 4.3
Author: 両毛トレード(両毛のビジネスノート)
*/

if (!defined('ABSPATH')) exit; // WordPress環境でのみ実行

// テーブル作成の関数
function rm_create_access_log_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE IF NOT EXISTS $table_name (
        id MEDIUMINT(9) NOT NULL AUTO_INCREMENT,
        ip_address VARCHAR(100) NOT NULL,
        visit_time DATETIME NOT NULL,
        referer_url TEXT,
        page_url TEXT,
        PRIMARY KEY (id)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}

// テーブルの存在チェック関数
function rm_check_table_exists() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';
    return ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") == $table_name);
}

// 管理メニューに「アクセスログ」メニューを追加
function rm_add_access_log_menu() {
    add_menu_page(
        'アクセスログ',
        'アクセスログ',
        'manage_options',
        'rm_access_log',
        'rm_access_log_page'
    );
}
add_action('admin_menu', 'rm_add_access_log_menu');

// 管理画面内でタブを切り替えるページ内容
// タブ追加に伴い、管理メニュー内のページ生成関数
// 管理画面内でタブを切り替えるページ内容
function rm_access_log_page() {
    $tab = isset($_GET['tab']) ? $_GET['tab'] : 'log';

    echo '<div class="wrap">';
    echo '<h1>アクセスログ</h1>';
    echo '<h2 class="nav-tab-wrapper">';
    echo '<a href="?page=rm_access_log&tab=log" class="nav-tab ' . ($tab == 'log' ? 'nav-tab-active' : '') . '">ログ表示</a>';
    echo '<a href="?page=rm_access_log&tab=summary" class="nav-tab ' . ($tab == 'summary' ? 'nav-tab-active' : '') . '">集計</a>';
    echo '<a href="?page=rm_access_log&tab=article" class="nav-tab ' . ($tab == 'article' ? 'nav-tab-active' : '') . '">ページ別アクセス数</a>';
    echo '<a href="?page=rm_access_log&tab=Search_Engine" class="nav-tab ' . ($tab == 'Search_Engine' ? 'nav-tab-active' : '') . '">検索エンジン流入データ</a>';
    echo '<a href="?page=rm_access_log&tab=top_ips" class="nav-tab ' . ($tab == 'top_ips' ? 'nav-tab-active' : '') . '">上位アクセスIP</a>';
    echo '</h2>';

    // 各タブに対応した関数を呼び出す
    if (!rm_check_table_exists()) {
        rm_create_table_page();
    } elseif ($tab == 'summary') {
        rm_display_summary();
    } elseif ($tab == 'article') {
        rm_display_article_access();
    } elseif ($tab == 'Search_Engine') {
        rm_display_Search_Engine(); // 検索エンジン流入
    } elseif ($tab == 'top_ips') {
        rm_display_top_ips(); // 上位アクセスIP表示
    } else {
        rm_display_access_log();
    }

    echo '</div>';
}




// 検索エンジン流入アクセス　2025/01/16
// 過去２週間のデータ
function rm_display_Search_Engine() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log'; // テーブル名

    // タイムゾーンを設定
    date_default_timezone_set('Asia/Tokyo');

    // 今日の日付と範囲を取得
    $today = current_time('Y-m-d');
    $start_date = date('Y-m-d', strtotime('-14 days', strtotime($today)));

    // 検索エンジン条件
    $search_engines = [
        'Google' => '%google.co%',
        'Yahoo' => '%yahoo.co%',
        'Bing' => '%bing.com%',
        'DuckDuckGo' => '%duckduckgo.com%',
        'Rakuten' => '%websearch.rakuten.co.jp%',
        'Baidu' => '%baidu.com%',
        'Oscobo' => '%oscobo.com%',
        'Docomo' => '%service.smt.docomo.ne.jp%',
        'GMO' => '%sp-search.jp%',
    ];

    // 日付ごとのアクセスデータを初期化
    $access_data = [];
    for ($i = 0; $i < 14; $i++) {
        $date = date('Y-m-d', strtotime("-$i days", strtotime($today)));
        $access_data[$date] = array_fill_keys(array_keys($search_engines), 0); // 初期値を0に
    }

    // 各検索エンジンごとのデータを取得
    foreach ($search_engines as $engine_name => $engine_url) {
        $query = $wpdb->prepare(
            "SELECT DATE(visit_time) AS log_date, COUNT(*) AS access_count
             FROM $table_name
             WHERE visit_time >= %s AND visit_time < %s
             AND referer_url LIKE %s
             GROUP BY DATE(visit_time)",
            $start_date, date('Y-m-d', strtotime('+1 day', strtotime($today))), $engine_url
        );

        $results = $wpdb->get_results($query, ARRAY_A);

        // 結果を日付ごとに格納
        foreach ($results as $row) {
            $log_date = $row['log_date'];
            if (isset($access_data[$log_date])) {
                $access_data[$log_date][$engine_name] = (int)$row['access_count'];
            }
        }
    }

    // 結果をテーブル形式で表示
    echo '<h2>過去14日間の検索エンジンアクセス数</h2>';
    echo '<table border="1" cellpadding="5" cellspacing="0">';
    echo '<tr><th>日付</th>';
    foreach (array_keys($search_engines) as $engine_name) {
        echo '<th>' . esc_html($engine_name) . '</th>';
    }
    echo '</tr>';

    foreach ($access_data as $date => $engine_data) {
        echo '<tr>';
        echo '<td>' . esc_html($date) . '</td>';
        foreach ($engine_data as $engine_name => $count) {
            // 明示的に0を含めてすべての列を出力
            echo '<td>' . esc_html($count) . '</td>';
        }
        echo '</tr>';
    }

    echo '</table>';
}






// 記事別アクセス数表示関数
function rm_display_article_access() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';

    // 日付フィルター設定
    $period = isset($_GET['period']) ? $_GET['period'] : 'today';
    $date_condition = '';

    switch ($period) {
        case '7days':
            $date_condition = "WHERE visit_time >= NOW() - INTERVAL 7 DAY";
            break;
        case '14days':
            $date_condition = "WHERE visit_time >= NOW() - INTERVAL 14 DAY";
            break;
        case '28days':
            $date_condition = "WHERE visit_time >= NOW() - INTERVAL 28 DAY";
            break;
        case '90days': //90対応　2024-12-4
            $date_condition = "WHERE visit_time >= NOW() - INTERVAL 90 DAY";
            break;			
        case '365days': //365対応　2024-12-4
            $date_condition = "WHERE visit_time >= NOW() - INTERVAL 365 DAY";
            break;
        default: // 今日
            $date_condition = "WHERE DATE(visit_time) = CURDATE()";
            break;
    }

    // フィルター付きのアクセス数集計
    $results = $wpdb->get_results("SELECT page_url, COUNT(*) as access_count FROM $table_name $date_condition GROUP BY page_url ORDER BY access_count DESC");

    // レスポンシブ対応のCSS
    echo '
    <style>
        /* デフォルトスタイル */
        .rm-date-filter-buttons {
            display: flex;
            gap: 10px;
            margin-bottom: 20px;
        }

        /* スマートフォン画面用 */
        @media (max-width: 600px) {
            .rm-date-filter-buttons {
                flex-direction: column;
                align-items: stretch;
            }
            .rm-date-filter-buttons button {
                width: 100%;
            }
        }
    </style>';

    // フィルターボタンの表示 90 365 追加　2024-12-4
    echo '<div class="rm-date-filter-buttons">';
    echo '<a href="?page=rm_access_log&tab=article&period=today"><button>今日</button></a>';
    echo '<a href="?page=rm_access_log&tab=article&period=7days"><button>7日</button></a>';
    echo '<a href="?page=rm_access_log&tab=article&period=14days"><button>14日</button></a>';
    echo '<a href="?page=rm_access_log&tab=article&period=28days"><button>28日</button></a>';
    echo '<a href="?page=rm_access_log&tab=article&period=90days"><button>90日</button></a>';
    echo '<a href="?page=rm_access_log&tab=article&period=365days"><button>365日</button></a>';
    echo '</div>';

    // アクセス数テーブルの表示
    echo '<h2>ページ別</h2>';
    echo '<table border=1><tr><th>URL</th><th>アクセス数</th></tr>';
    foreach ($results as $row) {
        echo '<tr>';
        echo '<td>' . esc_url($row->page_url) . '</td>';
        echo '<td>' . intval($row->access_count) . '</td>';
        echo '</tr>';
    }
    echo '</table>';
}





// 初回のみ表示される「テーブル作成」ページ内容
function rm_create_table_page() {
    if (isset($_POST['create_table'])) {
        rm_create_access_log_table();
        echo "<p>テーブルの作成が完了しました。</p>";
        return;
    }
    
    if (!rm_check_table_exists()) {
        echo '<h1>アクセスログ テーブル作成</h1>';
        echo '<form method="post"><input type="submit" name="create_table" value="テーブル作成"></form>';
    }
}

// アクセスログの集計データ表示
function rm_display_summary() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';

    // 現在のサイトURLのドメイン名を取得
    $site_domain = parse_url(home_url(), PHP_URL_HOST);  // 現在のドメイン名
	
	// リファラ無しデータのみ削除機能　20241213
	    if (isset($_POST['delete_direct_access'])) {
        // 直接アクセスデータの削除
        $wpdb->query("DELETE FROM $table_name WHERE referer_url = '直接アクセス'");
        echo "<p>直接アクセスデータを削除しました。</p>";
    }

    // SNS、検索エンジン、直接アクセス、外部リンク、内部リンクの集計
    // 主要検索エンジンに　oscobo　baiduを対応　2024-12-4
    // 主要検索エンジンに　duckduckgoを対応　2025-01-16
    $summary = [
        'sns' => [
            'X' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%t.co%'"),
            'Instagram' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%instagram.com%'"),
            'TikTok' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%tiktok.com%'"),
            'YouTube' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%youtube.com%'"),
            'Note' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%note.com%'"),
        ],
        'search_engines' => [
            'Google' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%google.co%'"),
            'Yahoo' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%yahoo.co%'"),
            'baidu' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%baidu.com%'"),
            'oscobo' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%oscobo.com%'"),
            'rakuten' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%websearch.rakuten.co.jp%'"),
            'duckduckgo' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%duckduckgo.com%'"),
            'docomo' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%service.smt.docomo.ne.jp%'"),
            'GMO' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%sp-search.jp%'"),
            'Bing' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url LIKE '%bing.com%'"),
        ],
        'direct_access' => $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE referer_url = '直接アクセス'"),

        // 外部リンク：検索エンジン、SNS、内部リンクを除外 httpが無いパターン追記　2024-12-4
        'external_links' => $wpdb->get_results("SELECT referer_url, COUNT(*) AS count FROM $table_name 
            WHERE referer_url NOT LIKE 'http://$site_domain%' 
            AND referer_url NOT LIKE 'https://$site_domain%' 
            AND referer_url NOT LIKE 'http://www.$site_domain%' 
            AND referer_url NOT LIKE 'https://www.$site_domain%' 
            AND referer_url != '$site_domain' 
            AND referer_url != '直接アクセス' 
            AND referer_url NOT LIKE '%t.co%' 
            AND referer_url NOT LIKE '%instagram.com%' 
            AND referer_url NOT LIKE '%tiktok.com%' 
            AND referer_url NOT LIKE '%youtube.com%' 
            AND referer_url NOT LIKE '%note.com%' 
            AND referer_url NOT LIKE '%google.com%' 
            AND referer_url NOT LIKE '%google.co.jp%' 
            AND referer_url NOT LIKE '%yahoo.co.jp%' 
            AND referer_url NOT LIKE '%yahoo.com%' 
            AND referer_url NOT LIKE '%baidu.com%' 
            AND referer_url NOT LIKE '%oscobo.com%' 
            AND referer_url NOT LIKE '%websearch.rakuten.co.jp%' 
            AND referer_url NOT LIKE '%duckduckgo.com%' 
            AND referer_url NOT LIKE '%service.smt.docomo.ne.jp%' 
            AND referer_url NOT LIKE '%sp-search.jp%' 
            AND referer_url NOT LIKE '%bing.com%' 
            GROUP BY referer_url ORDER BY count DESC LIMIT 10"),

        // 内部リンク：全パターンをカバー httpが無いパターン追記　2024-12-4
        'internal_links' => $wpdb->get_results("SELECT referer_url, COUNT(*) AS count FROM $table_name 
            WHERE referer_url LIKE 'http://$site_domain%' 
            OR referer_url LIKE 'https://$site_domain%' 
            OR referer_url = '$site_domain' 
            OR referer_url LIKE 'http://www.$site_domain%' 
            OR referer_url LIKE 'https://www.$site_domain%' 
            GROUP BY referer_url ORDER BY count DESC LIMIT 10"),
    ];

    // 表示
    echo '<h2>集計データ</h2>';
    echo '<h3>SNSの集客元</h3>';
    foreach ($summary['sns'] as $sns => $count) {
        echo "<p>$sns から：$count 件</p>";
    }

    echo '<h3>主要検索エンジン</h3>';
    foreach ($summary['search_engines'] as $engine => $count) {
        echo "<p>$engine から：$count 件</p>";
    }

    echo '<h3>直接アクセス(リファラ無し)</h3>';
    echo "<p>直接アクセス(リファラ無し)：{$summary['direct_access']} 件</p>";

// HTML部分に削除ボタン追加　20241213
echo '<form method="post">';
echo '<button type="submit" name="delete_direct_access">直接アクセスデータ削除</button>';
echo '</form>';

    echo '<h3>外部リンクからのアクセス（SNS・検索エンジン以外）</h3>';
    foreach ($summary['external_links'] as $link) {
        echo "<p>{$link->referer_url}：{$link->count} 件</p>";
    }

    echo '<h3>内部リンクからのアクセス</h3>';
    foreach ($summary['internal_links'] as $link) {
        echo "<p>{$link->referer_url}：{$link->count} 件</p>";
    }
}





// ページネーション付きのアクセスログ表示
function rm_display_access_log() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';

    $limit = 25;
    $paged = isset($_GET['paged']) ? (int)$_GET['paged'] : 1;
    $offset = ($paged - 1) * $limit;

    $total = $wpdb->get_var("SELECT COUNT(*) FROM $table_name");
    $results = $wpdb->get_results("SELECT * FROM $table_name ORDER BY visit_time DESC LIMIT $offset, $limit");

    echo '<h1>アクセスログ</h1>';
    echo '<table border=1><tr><th>ID</th><th>IPアドレス</th><th>訪問日時</th><th>参照元URL</th><th>アクセスページURL</th></tr>';
    foreach ($results as $row) {
        echo '<tr>';
        echo '<td>' . $row->id . '</td>';
        echo '<td>' . $row->ip_address . '</td>';
        echo '<td>' . $row->visit_time . '</td>';
        echo '<td>' . $row->referer_url . '</td>';
        echo '<td>' . $row->page_url . '</td>';
        echo '</tr>';
    }
    echo '</table>';

    $total_pages = ceil($total / $limit);
    for ($i = 1; $i <= $total_pages; $i++) {
        echo '<a href="?page=rm_access_log&tab=log&paged=' . $i . '" class="button">' . $i . '</a> ';
    }
}


// 上位20件の対応　2024/11/5
function rm_display_top_ips() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';

    // 削除処理が実行されたかをチェック
    if (isset($_POST['delete_ip'])) {
        $delete_ip = esc_sql($_POST['delete_ip']);
        $wpdb->query("DELETE FROM $table_name WHERE ip_address = '$delete_ip'");
        echo "<p>IPアドレス $delete_ip のログが削除されました。</p>";
    }

    // 上位20件のIPアドレス表示
    $top_ips = $wpdb->get_results("SELECT ip_address, COUNT(*) as access_count FROM $table_name GROUP BY ip_address ORDER BY access_count DESC LIMIT 20");

    // IPフィルターフォーム
    echo '<h2>IPによるアクセスフィルタ</h2>';
    echo '<form method="get" action="">';
    echo '<input type="hidden" name="page" value="rm_access_log">';
    echo '<input type="hidden" name="tab" value="top_ips">';
    echo 'IPアドレスを指定: <input type="text" name="filter_ip">';
    echo '<button type="submit">表示</button>';
    echo '</form>';

    // フィルタされたIPのアクセスログを取得して表示
    $filter_ip = isset($_GET['filter_ip']) ? esc_sql($_GET['filter_ip']) : '';
    if ($filter_ip) {
        $filtered_logs = $wpdb->get_results("SELECT * FROM $table_name WHERE ip_address = '$filter_ip' ORDER BY visit_time DESC");
        echo "<h3>IP: $filter_ip のアクセスログ</h3>";
        echo '<table border="1"><tr><th>日時</th><th>参照元URL</th><th>アクセス先URL</th></tr>';
        foreach ($filtered_logs as $log) {
            echo "<tr><td>{$log->visit_time}</td><td>{$log->referer_url}</td><td>{$log->page_url}</td></tr>";
        }
        echo '</table>';

        // 削除ボタン
        echo '<form method="post" action="">';
        echo '<input type="hidden" name="delete_ip" value="' . esc_attr($filter_ip) . '">';
        echo '<button type="submit">指定IPのログを削除</button>';
        echo '</form>';
    }

    // 上位20件の表示
    echo '<h2>上位アクセスIP（上位20件）</h2>';
    echo '<table border="1"><tr><th>IPアドレス</th><th>アクセス数</th></tr>';
    foreach ($top_ips as $ip) {
        echo "<tr><td>{$ip->ip_address}</td><td>{$ip->access_count}</td></tr>";
    }
    echo '</table>';
	
	
// 全削除ボタンを表示するコード 20241115追記
if (isset($_POST['delete_all_logs'])) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';
    $wpdb->query("TRUNCATE TABLE $table_name"); // 全データ削除

    // 削除完了メッセージを表示
    echo "<div style='margin: 10px; padding: 10px; background-color: #f0f0f0; border: 1px solid #ccc;'>
             すべてのログが削除されました。
          </div>";
}

echo "<form method='post' style='margin-top: 20px;'>
          <button type='submit' name='delete_all_logs' style='padding: 10px 20px; background-color: #ff4444; color: #fff; border: none; cursor: pointer;'
                  onclick=\"return confirm('本当にすべてのログを削除しますか？');\">
              すべてのログを削除
          </button>
      </form>";

	
}


// IPログ削除機能
if (isset($_POST['delete_ip'])) {
    $delete_ip = esc_sql($_POST['delete_ip']);
    $wpdb->query("DELETE FROM $table_name WHERE ip_address = '$delete_ip'");
    echo "<p></p>";
}


// アクセスログの記録（フッターにて記録）
function rm_log_visit() {
    if (is_admin()) return; // 管理画面での記録を防ぐ

    global $wpdb;
    $table_name = $wpdb->prefix . 'rm_access_log';
    $ip_address = $_SERVER['REMOTE_ADDR'];
    $referer_url = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '直接アクセス';
    $page_url = home_url($_SERVER['REQUEST_URI']);

    // クローラーのユーザーエージェントパターン
    $crawler_agents = [
        'Googlebot', 'Bingbot', 'Slurp', 'DuckDuckBot', 'Baiduspider', 'YandexBot', 'Sogou',
        'Exabot', 'facebot', 'ia_archiver', 'SemrushBot', 'AhrefsBot', 'MJ12bot', 'DotBot',
        'BLEXBot', 'UptimeRobot', 'Applebot', 'googlezip.net', 'amazonaws.com', 'search.msn.com', 'leakix.org'
    ];

    // ユーザーエージェントをチェックし、クローラーの場合は記録しない
    $user_agent = $_SERVER['HTTP_USER_AGENT'];
    foreach ($crawler_agents as $agent) {
        if (stripos($user_agent, $agent) !== false) {
            return; // クローラーアクセスを除外
        }
    }

    // 除外するファイルの拡張子リスト
    $exclude_extensions = ['jpg', 'jpeg', 'png', 'gif', 'svg', 'css', 'js', 'ico'];

    // URLの拡張子をチェックして除外
    $path_info = pathinfo(parse_url($page_url, PHP_URL_PATH));
    if (isset($path_info['extension']) && in_array(strtolower($path_info['extension']), $exclude_extensions)) {
        return; // 画像やCSS、JSファイルへのアクセスを除外
    }

    // 管理者のアクセスは除外
    if (current_user_can('administrator')) return;

    // アクセスログを記録
    $wpdb->insert(
        $table_name,
        array(
            'ip_address' => $ip_address,
            'visit_time' => current_time('mysql'), // 現在時刻を設定
            'referer_url' => $referer_url,
            'page_url' => $page_url,
        )
    );
}
add_action('wp_footer', 'rm_log_visit');

